<?php
/**
* 2019-2022 BIZAO
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    BIZAO <contact@bizao.com>
*  @copyright 2019-2022 BIZAO
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class BizaoPay extends PaymentModule
{

    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    public function __construct()
    {
        $this->module_key = '9e8e8ea21101c08a093e7292a1f803b4';
        $this->name = 'bizaopay';
        $this->tab = 'payments_gateways';
        $this->version = '1.1.0';
        $this->ps_versions_compliancy = array('min' => '1.6.0.4', 'max' => _PS_VERSION_);
        $this->author = 'Bizao';

        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $this->controllers = array('payment', 'validation');
        } else {
            $this->controllers = array('validation');
        }
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('BizaoPay​');
        $this->description = $this->l('Pay easily and securely with your Mobile Money account or your bank card');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install()
    {
        //include sql file to create tables
        require_once(dirname(__FILE__).'/sql/install.php');
        if (!parent::install()
            or !$this->registerHook('paymentOptions')
            or !$this->registerHook('paymentReturn')
            or !Configuration::updateValue('BIZAOPAY_TEXT', 'Payez avec votre compte Mobile Money​ ou votre carte bancaire')
            or !Configuration::updateValue('BIZAOPAY_DESCRIPTION', ' Effectuez le paiement de votre commande avec votre compte Mobile Money ou votre carte bancaire.​')
            or !Configuration::updateValue('BIZAOPAY_CLIENT_ID', '')
            or !Configuration::updateValue('BIZAOPAY_SECRET_KEY', '')

            or !Configuration::updateValue('BIZAOPAY_CLIENT_ID_TEST', '')
            or !Configuration::updateValue('BIZAOPAY_SECRET_KEY_TEST', '')

            or !Configuration::updateValue('BIZAOPAY_STATE', '')
            or !Configuration::updateValue('BIZAOPAY_REFERENCE', '')

            or !Configuration::updateValue('BIZAOPAY_NOTIF_URL', '')
            or !Configuration::updateValue('BIZAOPAY_LIVE_MODE', false)
            or !Configuration::updateValue('BIZAOPAY_SUCCESS_MESSAGE', 'Paiement effectué avec succès !​')
            or !Configuration::updateValue('BIZAOPAY_ERROR_MESSAGE', 'Votre paiement n’a pas abouti​')
            or !Configuration::updateValue('BIZAOPAY_PENDING_STATE', 0)

            or !Configuration::updateValue('BIZAOPAY_LIVE_URL', "https://secure.bizao.com/entry")
            or !Configuration::updateValue('BIZAOPAY_SANDBOX_URL', "https://d36ikzw287ira.cloudfront.net/#/entry")
          ) {
            return false;
        }
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $this->registerHook('payment');
        }

        if (!$this->installOrderState()) {
            return false;
        }
        return true;
    }
    /**
     * Create order state
     * @return boolean
     */
    public function installOrderState()
    {
        if (!Configuration::get('BIZAO_OS_WAITING')
            || !Validate::isLoadedObject(new OrderState(Configuration::get('BIZAO_OS_WAITING')))) {
            $order_state = new OrderState();
            $order_state->name = array();
            foreach (Language::getLanguages() as $language) {
                if (Tools::strtolower($language['iso_code']) == 'fr') {
                    $order_state->name[$language['id_lang']] = 'En attente du paiement par Mobile Money';
                } else {
                    $order_state->name[$language['id_lang']] = 'Awaiting for mobile money payment';
                }
            }
            $order_state->send_email = false;
            $order_state->color = '#4169E1';
            $order_state->hidden = false;
            $order_state->delivery = false;
            $order_state->logable = false;
            $order_state->invoice = false;
            $order_state->module_name = $this->name;
            if ($order_state->add()) {
                $source = _PS_MODULE_DIR_ . 'bizaopay/views/img/os_bizao.png';
                $destination = _PS_ROOT_DIR_ . '/img/os/' . (int)$order_state->id . '.gif';
                copy($source, $destination);
            }

            if (Shop::isFeatureActive()) {
                $shops = Shop::getShops();
                foreach ($shops as $shop) {
                    Configuration::updateValue('BIZAO_OS_WAITING', (int) $order_state->id, false, null, (int)$shop['id_shop']);
                }
            } else {
                Configuration::updateValue('BIZAO_OS_WAITING', (int) $order_state->id);
            }
        }

        return true;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $this->context->smarty->assign(array(
            'bizaopay_description' => Configuration::get('BIZAOPAY_DESCRIPTION')
        ));

        $payment_options = [
            $this->getPaymentOption(),
        ];

        return $payment_options;
    }

    /*For Prestashop < 1.7 */
    public function hookPayment($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $this->smarty->assign(array(
            'this_path' => $this->_path,
            'this_path_bw' => $this->_path,
            'bizaopay_description' => Configuration::get('BIZAOPAY_DESCRIPTION'),
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
        ));
        return $this->display(__FILE__, 'payment_infos_'.$this->context->language->iso_code.'.tpl');
    }

    public function hookPaymentReturn($params)
    {
        $controller = $this->getHookController('paymentReturn');
        return $controller->run($params);
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /*For Prestashop >= 1.7*/
    public function getPaymentOption()
    {
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText(Configuration::get('BIZAOPAY_TEXT'))
                      ->setAction($this->context->link->getModuleLink($this->name, 'payment', array(), true))
                      ->setAdditionalInformation($this->context->smarty->fetch('module:bizaopay/views/templates/front/payment_infos_'.$this->context->language->iso_code.'.tpl'));

        return $option;
    }


    public function getContent()
    {
        $controller = $this->getHookController('getContent');
        return $controller->run();
    }


    public function getHookController($hook_name)
    {
        // Include the controller file
        require_once(dirname(__FILE__).'/controllers/hook/'. $hook_name.'.php');

        // Build dynamically the controller name
        $controller_name = 'Bizaopay'.$hook_name.'Controller';

        // Instantiate controller
        $controller = new $controller_name($this, __FILE__, $this->_path);

        // Return the controller
        return $controller;
    }


    public function uninstall()
    {
        //include sql file to create tables
        require_once(dirname(__FILE__).'/sql/uninstall.php');
        Configuration::deleteByName('BIZAOPAY_TEXT');
        Configuration::deleteByName('BIZAOPAY_DESCRIPTION');
        Configuration::deleteByName('BIZAOPAY_CLIENT_ID');
        Configuration::deleteByName('BIZAOPAY_SECRET_KEY');
        Configuration::deleteByName('BIZAOPAY_CLIENT_ID_TEST');
        Configuration::deleteByName('BIZAOPAY_SECRET_KEY_TEST');
        Configuration::deleteByName('BIZAOPAY_NOTIF_URL');
        Configuration::deleteByName('BIZAOPAY_SUCCESS_MESSAGE');
        Configuration::deleteByName('BIZAOPAY_ERROR_MESSAGE');
        Configuration::deleteByName('BIZAOPAY_PENDING_STATE');
        Configuration::deleteByName('BIZAOPAY_STATE');
        Configuration::deleteByName('BIZAOPAY_REFERENCE');
        Configuration::deleteByName('BIZAOPAY_NOTIF_URL');
        Configuration::deleteByName('BIZAO_OS_WAITING');
        Configuration::deleteByName('BIZAOPAY_LIVE_URL');
        Configuration::deleteByName('BIZAOPAY_SANDBOX_URL');
        return parent::uninstall();
    }
}
