<?php
/**
* 2019-2022 BIZAO
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    BIZAO <contact@bizao.com>
*  @copyright 2019-2022 BIZAO
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class BizaopayPaymentModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    public $order_id;
    public $currency;
    public $amount;
    public $payment_url = "";
    protected $encryptMethod = 'AES-256-CBC';

    public function initContent()
    {

            // Call parent init content method
        parent::initContent();
        if (Configuration::get('BIZAOPAY_LIVE_MODE')) {
            $this->payment_url = Configuration::get('BIZAOPAY_LIVE_URL');
        } else {
            $this->payment_url = Configuration::get('BIZAOPAY_SANDBOX_URL');
        }

        // Check if currency is accepted
        if (!$this->checkCurrency()) {
            Tools::redirect('index.php?controller=order');
        }

        // Check if cart exists and all fields are set
        $cart = $this->context->cart;
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        // Check if module is enabled
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == $this->module->name) {
                $authorized = true;
            }
        }
        if (!$authorized) {
            die('This payment method is not available.');
        }


        // Check if customer exists
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $this->getPaymentArgs();
    }

    private function checkCurrency()
    {
        // Get cart currency and enabled currencies for this module
        $currency_order = new Currency($this->context->cart->id_currency);
        $currencies_module = $this->module->getCurrency($this->context->cart->id_currency);

        // Check if cart currency is one of the enabled currencies
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        // Return false otherwise
        return false;
    }


    private function getPaymentArgs()
    {
        $all_lang = Language::getLanguages();

        $this->currency = $this->context->currency->iso_code;
        $this->amount = (float)$this->context->cart->getOrderTotal(true, Cart::BOTH);
        $state = Configuration::get('BIZAOPAY_STATE');
        $reference = Configuration::get('BIZAOPAY_REFERENCE');
        $lang = Tools::strtolower($all_lang[0]['iso_code']);
        $client_id = Configuration::get('BIZAOPAY_LIVE_MODE') ? Configuration::get('BIZAOPAY_CLIENT_ID') : Configuration::get('BIZAOPAY_CLIENT_ID_TEST');
        $notif_url = Tools::substr(Configuration::get('BIZAOPAY_NOTIF_URL'), 0, 3) == 'htt' ? Configuration::get('BIZAOPAY_NOTIF_URL') : 'https://webapi.bizao.com/api/mobilemoney/transaction/notify';

        $bizao_reference = $this->getBizaoReference();
        $bizao_module = "Bizaopay";
        $bizao_state = "paiement";
        $bizao_id_cart = $this->context->cart->id;

        $sq =  'TRUNCATE TABLE `' . _DB_PREFIX_ . 'bizaopay_order`';

        Db::getInstance()->execute($sq);

        $sql = array( "reference" => $bizao_reference, "state" => $bizao_state, "module" => $bizao_module, "amount" => round($this->amount, 0), "id_cart" => $bizao_id_cart);
        Db::getInstance()->insert('bizaopay_order', $sql);

        $return_url = $this->context->link->getModuleLink('bizaopay', 'validation', array('bizao_o_ref' => $bizao_reference), Configuration::get('PS_SSL_ENABLED'));

        //build array for encrypt
        $obj_args = array(
              'currency'          => $this->currency,
              'order_id'          => Tools::substr(Tools::strtoupper($bizao_reference.'_ps_'.$reference), 0, 25),
              'amount'            => round($this->amount, 0),
              'state'             => $state,
              'reference'         => $reference,
              'lang'              => $lang,
              'client_id'         => $client_id,
              'return_url'        => $return_url,
              'cancel_url'        => $return_url,
              'notif_url'         => $notif_url,
              'country'           => $this->context->country->iso_code,
            );

        //encode to string
        $json_args = json_encode($obj_args);
        //encode string args
        $session_key = $this->encryptWithRSA($json_args);

        //redirect to payment page
        Tools::redirectLink($this->payment_url.'?sessionId='.urlencode($session_key));
    }

    public function encryptWithRSA($source)
    {
        try {
            //path holds the certificate path present in the system
            $path = Tools::getShopDomainSsl(true, true).__PS_BASE_URI__."modules/bizaopay/pki/public.crt";
            $public_key = Tools::file_get_contents($path);

            $start = 0; //initial character start
                $length = Tools::strlen($source)/50; // length of data divided by 10
                $num_length = floor($length); //arround length to small number
                $crt = "";
            for ($i=0; $i < $num_length; $i++) {
                $crypttext = $this->rsaEncrypter(Tools::substr($source, $start, 50), openssl_pkey_get_public($public_key));
                $start+= 50;  // increment start
                  $crt.= $crypttext; //concat with encrypted data
                  $crt.= ":::"; //breakpoint
            }

            //prevent floot result  on division
            if ((Tools::strlen($source)%50)>0) {
                $crypttext = $this->rsaEncrypter(Tools::substr($source, $start), openssl_pkey_get_public($public_key));
                $crt.= $crypttext; //concat with encrypted data
            }
            return base64_encode($crt); //encoding on base64
        } catch (\Exception $e) {
            var_dump($e->getMessage());
            die();
        }
    }

    public function rsaEncrypter($plaintext, $public_key)
    {
        $invalidcipher = true;
        $crypttext = '';
        do {
            openssl_public_encrypt($plaintext, $crypttext, $public_key); // encrypt every section of data
            if (Tools::substr(base64_encode($crypttext), -2) == "==") {
                $invalidcipher = false;
            } else {
                $crypttext = '';
                $invalidcipher = true;
            }
        } while ($invalidcipher);

        return $crypttext;
    }

    /**
    * Get all datas from Order
    */
    public function getBizaoOrderId()
    {
        $sql = 'SELECT `id_order`
          FROM `'._DB_PREFIX_.'orders`
          ORDER BY `id_order` DESC';
        $result = Db::getInstance()->getRow($sql);
        if ($result != false) {
            return ((int) $result['id_order']) + 1;
        }
        return 0;
    }

    public function getBizaoReference()
    {
        $length = 9;
        $duplicate = true;
        do {
            $reference = Tools::substr(str_shuffle("ABCDEFGHIJKLMNOPQRSTWXYZ"), 0, $length);
            $sql = 'SELECT `reference`
            FROM `'._DB_PREFIX_.'bizaopay_order`
            WHERE `reference` = \''.pSQL($reference).'\'';
            $result = Db::getInstance()->getRow($sql);
            if ($result['reference']) {
                $duplicate = true;
            } else {
                $duplicate = false;
            }
        } while ($duplicate);

        return $reference;
    }
}
