<?php
/**
* 2019-2022 BIZAO
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    BIZAO <contact@bizao.com>
*  @copyright 2019-2022 BIZAO
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class BizaopayGetContentController
{
    protected $html = '';
    protected $postErrors = array();

    public function __construct($module, $file, $path)
    {
        $this->file = $file;
        $this->module = $module;
        $this->context = Context::getContext();
        $this->_path = $path;
    }

    protected function postValidation()
    {
        if (Tools::isSubmit('bizaopay_form')) {
            if (!Tools::getValue('BIZAOPAY_CLIENT_ID')) {
                $this->postErrors[] = $this->module->l('CLIENT ID Live is required.');
            }

            if (!Tools::getValue('BIZAOPAY_SECRET_KEY')) {
                $this->postErrors[] = $this->module->l('CLIENT SECRET Live is required.');
            }

            if (!Tools::getValue('BIZAOPAY_CLIENT_ID_TEST')) {
                $this->postErrors[] = $this->module->l('CLIENT ID Test is required.');
            }

            if (!Tools::getValue('BIZAOPAY_SECRET_KEY_TEST')) {
                $this->postErrors[] = $this->module->l('CLIENT SECRET Test is required.');
            }

            if (!Tools::getValue('BIZAOPAY_TEXT')) {
                $this->postErrors[] = $this->module->l('Payment button text is required.');
            }

            if (!Tools::getValue('BIZAOPAY_DESCRIPTION')) {
                $this->postErrors[] = $this->module->l('Payment button description is required.');
            }

            if (!Tools::getValue('BIZAOPAY_STATE')) {
                $this->postErrors[] = $this->module->l('State field is required.');
            }

            if (!Tools::getValue('BIZAOPAY_REFERENCE')) {
                $this->postErrors[] = $this->module->l('Reference field is required.');
            }

            if (!Tools::getValue('BIZAOPAY_SUCCESS_MESSAGE')) {
                $this->postErrors[] = $this->module->l('Define a success message please.');
            }

            if (!Tools::getValue('BIZAOPAY_ERROR_MESSAGE')) {
                $this->postErrors[] = $this->module->l('Define an error message please.');
            }
        }
    }

    public function processConfiguration()
    {
        if (Tools::isSubmit('bizaopay_form')) {
            $this->postValidation();
            if (!count($this->postErrors)) {
                Configuration::updateValue('BIZAOPAY_LIVE_MODE', Tools::getValue('BIZAOPAY_LIVE_MODE'));
                Configuration::updateValue('BIZAOPAY_CLIENT_ID', Tools::getValue('BIZAOPAY_CLIENT_ID'));
                Configuration::updateValue('BIZAOPAY_SECRET_KEY', Tools::getValue('BIZAOPAY_SECRET_KEY'));
                Configuration::updateValue('BIZAOPAY_CLIENT_ID_TEST', Tools::getValue('BIZAOPAY_CLIENT_ID_TEST'));
                Configuration::updateValue('BIZAOPAY_SECRET_KEY_TEST', Tools::getValue('BIZAOPAY_SECRET_KEY_TEST'));
                Configuration::updateValue('BIZAOPAY_TEXT', Tools::getValue('BIZAOPAY_TEXT'));
                Configuration::updateValue('BIZAOPAY_DESCRIPTION', Tools::getValue('BIZAOPAY_DESCRIPTION'));
                Configuration::updateValue('BIZAOPAY_SUCCESS_MESSAGE', Tools::getValue('BIZAOPAY_SUCCESS_MESSAGE'));
                Configuration::updateValue('BIZAOPAY_ERROR_MESSAGE', Tools::getValue('BIZAOPAY_ERROR_MESSAGE'));

                Configuration::updateValue('BIZAOPAY_STATE', Tools::getValue('BIZAOPAY_STATE'));
                Configuration::updateValue('BIZAOPAY_REFERENCE', Tools::getValue('BIZAOPAY_REFERENCE'));

                Configuration::updateValue('BIZAOPAY_NOTIF_URL', Tools::getValue('BIZAOPAY_NOTIF_URL'));

                $this->context->smarty->assign('confirmation', 'ok');
            } else {
                foreach ($this->postErrors as $err) {
                    $this->context->smarty->assign(array(
                           'error_text' => $err
                       ));
                }
            }
        }
    }

    public function renderForm()
    {
        $this->html = '<div class="alert alert-warning">' . $this->module->l("In order to complete the activation of the plugin, please fill in the form on this link.") . '<br/>'  . '<a target="_blank" href="https://bizao.com/demarrer-un-projet/?utm_source=Plugins&utm_medium=organic&utm_campaign=Prestashop">Démarrer un projet - Bizao</a> <br>
           <p> Une documentation faisant office de KYC sera obligatoire pour l’utilisation du plugin. </p> </div>';
        $this->html .= '<div class="row" style="background: #fff; padding: 10px 20px; border-radius: 5px; margin: 10px 0 15px 0;">
                    <h4>Ressources utiles</h4>
                    <p style="margin-bottom: 20px;"> <b>NB:</b> Vous pouvez obtenir vos identifiants depuis votre interface <a target="_blank" href="https://bizao.com">BSC</a> ou votre <a target="_blank" href="https://sandbox.bizao.com">sandbox</a>  </p>
                    <div class="col-lg-4">
                      <a target="_blank" href="https://dev.bizao.com">Documentation Technique</a>
                    </div>
                    <div class="col-lg-4">
                      <a target="_blank" href="https://dev.bizao.com">Support Technique</a>
                    </div>
                    <div class="col-lg-4">
                      <a target="_blank" href="https://dev.bizao.com">Support Commercial</a>
                    </div>
                  </div>';

        $inputs = array(
                    array(
                          'type'    => 'switch',
                          'label'   => $this->module->l('Production mode '),
                          'name'    => 'BIZAOPAY_LIVE_MODE',
                          'is_bool' => true,
                          'desc'    => $this->module->l('Choose on which environment you use the plugin'),
                          'values' => array(
                              array(
                                  'id'    => 'active_on',
                                  'value' => true,
                                  'label' => $this->module->l('Production'),
                              ),
                              array(
                                  'id' => 'active_off',
                                  'value' => false,
                                  'label' => $this->module->l('Test')
                              )
                          ),
                      ),
                      array('type' => 'text',  'desc' => $this->module->l('The Live Client ID is your production client ID provided by BIZAO'), 'name' => 'BIZAOPAY_CLIENT_ID', 'label' => $this->module->l('Client ID Live'), 'required' => 'true'),
                      array('type' => 'text',  'desc' => $this->module->l('The Sceret Live Key is your secret production ID provided by BIZAO'), 'name' => 'BIZAOPAY_SECRET_KEY', 'label' => $this->module->l('Secret Key Live'),  'required' => 'true'),

                      array('type' => 'text',  'desc' => $this->module->l('The Test Client ID is your client ID provided by BIZAO on test environment'), 'name' => 'BIZAOPAY_CLIENT_ID_TEST', 'label' => $this->module->l('Client ID Test'), 'required' => 'true'),
                      array('type' => 'text',  'desc' => $this->module->l('The Sceret test Key is your secret ID provided by BIZAO​ on test environment'), 'name' => 'BIZAOPAY_SECRET_KEY_TEST', 'label' => $this->module->l('Secret Key Test'),  'required' => 'true'),

                      array('type' => 'text',  'desc' => $this->module->l('Text that the customer will see when paying for their order'), 'name' => 'BIZAOPAY_TEXT', 'label' => $this->module->l('Payment button text'), 'required' => 'true'),
                      array('type' => 'textarea',  'desc' => $this->module->l('​Description of the payment methods that the user will see under the Payment Button.'), 'name' => 'BIZAOPAY_DESCRIPTION', 'label' => $this->module->l('Description of the payment button'), 'required' => 'true'),
                      array('type' => 'text',  'desc' => $this->module->l('The Reference that has been provided to you'), 'name' => 'BIZAOPAY_REFERENCE', 'label' => $this->module->l('Reference'), 'required' => 'true'),
                      array('type' => 'text',  'desc' => $this->module->l('The state that has been provided to you'), 'name' => 'BIZAOPAY_STATE', 'label' => $this->module->l('State'), 'required' => 'true'),

                      array('type' => 'text',  'desc' => $this->module->l('Transaction status notification URL'), 'name' => 'BIZAOPAY_NOTIF_URL', 'label' => $this->module->l('Notification URL')),

                      array('type' => 'textarea',  'desc' => $this->module->l('Message that the customer will see after placing the order.'), 'name' => 'BIZAOPAY_SUCCESS_MESSAGE', 'label' => $this->module->l('Message displayed on successful payment'), 'required' => 'true'),
                      array('type' => 'textarea',  'desc' => $this->module->l('Message that the customer will see if for some reason the transaction could not be completed.'), 'name' => 'BIZAOPAY_ERROR_MESSAGE', 'label' => $this->module->l('Message displayed in case of failed payment'), 'required' => 'true'),
          );

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->module->l('Configuration Bizaopay'),
                    'icon' => 'icon-wrench'
                ),
                'input' => $inputs,
                'submit' => array('title' => $this->module->l('Save'))
            )
        );

        $helper = new HelperForm();
        $helper->table = 'bizaopay';
        $helper->default_form_language = (int)Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int)Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->submit_action = 'bizaopay_form';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->module->name.'&tab_module='.$this->module->tab.'&module_name='.$this->module->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => array(
              'BIZAOPAY_LIVE_MODE' => Tools::getValue('BIZAOPAY_LIVE_MODE', Configuration::get('BIZAOPAY_LIVE_MODE')),
              'BIZAOPAY_CLIENT_ID' => Tools::getValue('BIZAOPAY_CLIENT_ID', Configuration::get('BIZAOPAY_CLIENT_ID')),
              'BIZAOPAY_SECRET_KEY' => Tools::getValue('BIZAOPAY_SECRET_KEY', Configuration::get('BIZAOPAY_SECRET_KEY')),
              'BIZAOPAY_CLIENT_ID_TEST' => Tools::getValue('BIZAOPAY_CLIENT_ID_TEST', Configuration::get('BIZAOPAY_CLIENT_ID_TEST')),
              'BIZAOPAY_SECRET_KEY_TEST' => Tools::getValue('BIZAOPAY_SECRET_KEY_TEST', Configuration::get('BIZAOPAY_SECRET_KEY_TEST')),
              'BIZAOPAY_TEXT' => Tools::getValue('BIZAOPAY_TEXT', Configuration::get('BIZAOPAY_TEXT')),
              'BIZAOPAY_DESCRIPTION' => Tools::getValue('BIZAOPAY_DESCRIPTION', Configuration::get('BIZAOPAY_DESCRIPTION')),
              'BIZAOPAY_STATE' => Tools::getValue('BIZAOPAY_STATE', Configuration::get('BIZAOPAY_STATE')),
              'BIZAOPAY_REFERENCE' => Tools::getValue('BIZAOPAY_REFERENCE', Configuration::get('BIZAOPAY_REFERENCE')),
              'BIZAOPAY_SUCCESS_MESSAGE' => Tools::getValue('BIZAOPAY_SUCCESS_MESSAGE', Configuration::get('BIZAOPAY_SUCCESS_MESSAGE')),
              'BIZAOPAY_ERROR_MESSAGE' => Tools::getValue('BIZAOPAY_ERROR_MESSAGE', Configuration::get('BIZAOPAY_ERROR_MESSAGE')),
              'BIZAOPAY_NOTIF_URL' => Tools::getValue('BIZAOPAY_NOTIF_URL', Configuration::get('BIZAOPAY_NOTIF_URL')),
            ),
            'languages' => $this->context->controller->getLanguages()
        );

        $this->html .= $helper->generateForm(array($fields_form));

        return $this->html;
    }

    public function run()
    {
        $this->processConfiguration();
        $html_confirmation_message = $this->module->display($this->file, 'getContent.tpl');
        $html_form = $this->renderForm();
        return $html_confirmation_message.$html_form;
    }
}
