<?php
/**
* 2019-2022 BIZAO
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    BIZAO <contact@bizao.com>
*  @copyright 2019-2022 BIZAO
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class BizaopayPaymentReturnController
{
    public function __construct($module, $file, $path)
    {
        $this->file = $file;
        $this->module = $module;
        $this->context = Context::getContext();
        $this->_path = $path;
    }

    public function run()
    {
        /*
        * If the module is not active anymore, no need to process anything.
        */
        if ($this->module->active == false) {
            Tools::redirect('index.php?controller=order&step=2');
        }

        try {
            if (empty(Tools::getValue('mno_name')) || empty(Tools::getValue('country_code')) || empty(Tools::getValue('bizao_o_ref'))) {
                Tools::redirect('index.php?controller=order');
            } else {
                $this->context = Context::getContext();
                $shop = new Shop(Configuration::get('PS_SHOP_DEFAULT'));
                Tools::getShopProtocol().$shop->domain.$shop->getBaseURI();

                $token_url = "https://api.bizao.com/token";
                $char = Configuration::get('BIZAOPAY_LIVE_MODE') ? Configuration::get('BIZAOPAY_CLIENT_ID').':'.Configuration::get('BIZAOPAY_SECRET_KEY') : Configuration::get('BIZAOPAY_CLIENT_ID_TEST').':'.Configuration::get('BIZAOPAY_SECRET_KEY_TEST');
                //first get TOKEN
                $authorizationKey = base64_encode($char);
                $req = curl_init();
                curl_setopt($req, CURLOPT_URL, $token_url);
                curl_setopt($req, CURLOPT_HEADER, 0);
                curl_setopt($req, CURLOPT_NOBODY, false); // remove body
                curl_setopt($req, CURLOPT_RETURNTRANSFER, 1);

                curl_setopt(
                    $req,
                    CURLOPT_HTTPHEADER,
                    array(
                          "authorization: Basic $authorizationKey",
                          "Content-Type: application/x-www-form-urlencoded"
                      )
                );
                curl_setopt($req, CURLOPT_POST, true);
                curl_setopt($req, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
                $response = curl_exec($req);

                $resArr = array();
                $resArr = json_decode($response);
                $access_token = $resArr->access_token;

                //get status
                $reqStatus = curl_init();
                $mno_name = Tools::getValue('mno_name');
                $country_code = Tools::getValue('country_code');

                curl_setopt($reqStatus, CURLOPT_HEADER, 0);
                curl_setopt($reqStatus, CURLOPT_NOBODY, false); // remove body
                curl_setopt($reqStatus, CURLOPT_RETURNTRANSFER, 1);

                if (Tools::getValue('mno_name') == 'vmc') {
                    $status_url = "https://api.bizao.com/debitCard/v2/getStatus/".Tools::getValue('order_reference');
                    curl_setopt($reqStatus, CURLOPT_URL, $status_url);
                    curl_setopt(
                        $reqStatus,
                        CURLOPT_HTTPHEADER,
                        array(
                            "Authorization: Bearer $access_token",
                            "Content-Type: application/json",
                            "country-code: $country_code"
                        )
                    );
                } else {
                    $status_url = "https://api.bizao.com/mobilemoney/getstatus/v1/".Tools::getValue('order_reference');
                    curl_setopt($reqStatus, CURLOPT_URL, $status_url);
                    curl_setopt(
                        $reqStatus,
                        CURLOPT_HTTPHEADER,
                        array(
                            "Authorization: Bearer $access_token",
                            "Content-Type: application/json",
                            "country-code: $country_code",
                            "mno-name: $mno_name",
                            "channel: web"
                        )
                    );
                }

                $resp = curl_exec($reqStatus);

                $res = array();
                $res = json_decode($resp);

                //update order state
                if (Tools::strtolower($res->status) == 'success' || Tools::strtolower($res->status) == 'successfull' || Tools::strtolower($res->status) == 'successful') {
                    $this->updateOrderAndRedirect();
                } else {
                    $this->context->smarty->assign('return_message', Configuration::get('BIZAOPAY_ERROR_MESSAGE'));
                    Tools::redirect('index.php?controller=order');
                }
            }
        } catch (\Exception $e) {
            Tools::redirect('index.php?controller=order');
        }
    }

    public function updateOrderAndRedirect()
    {
        $this->validateOrder();
        $order_id = (int)Order::getOrderByCartId($this->context->cart->id);
        $order = new Order($order_id);
        $order->setCurrentState(Configuration::get('PS_OS_PAYMENT'));

        $this->updateReference($order_id);

        $this->context->smarty->assign('return_message', Configuration::get('BIZAOPAY_SUCCESS_MESSAGE'));
        Tools::redirectLink(__PS_BASE_URI__ . 'index.php?controller=order-detail&id_order='.$order->id);
    }

    protected function validateOrder()
    {
        try {
            $sql = 'SELECT `amount`
                FROM `'._DB_PREFIX_.'bizaopay_order`
                WHERE `reference` = \''.pSQL(Tools::getValue('bizao_o_ref')).'\'';
            $res = Db::getInstance()->getRow($sql);

            $extra_vars = array(
            '{total_to_pay}' => Tools::displayPrice($res['amount'])
            );

            $this->module->validateOrder(
                $this->context->cart->id,
                Configuration::get('BIZAO_OS_WAITING'),
                $res['amount'],
                $this->module->displayName,
                null,
                $extra_vars,
                (int)$this->context->currency->id,
                false,
                $this->context->cart->secure_key
            );
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }

    public function updateReference($order_id)
    {
        try {
            $result = Db::getInstance()->update('orders', array(
              'reference' => pSQL(Tools::getValue('bizao_o_ref'))
            ), 'id_order = '.(int)$order_id, 1, true);
            return $result;
        } catch (\Exception $e) {
            return false;
        }
    }
}
